#include <vector>
#include <iostream>
#include <set>
#include <map>
#include <algorithm>

using namespace std;

typedef long long ll;
typedef vector<ll> vll;
typedef vector<vll> vvll;

struct Node {
    int mx;

    Node(int mx = 0) : mx(mx) {}

    Node operator+(const Node &node) const {
        return max(mx, node.mx);
    }
};

struct SegmentTree {
    vector<Node> data;
    vector<int> base;

    SegmentTree(vector<int> &base) : base(base), data(4 * (int) base.size()) {}

    void update(int v, int l, int r, int p, int x) {
        if (l + 1 == r) {
            data[v] = Node(x);
            return;
        }
        int m = (l + r) / 2;
        if (p < m) {
            update(2 * v + 1, l, m, p, x);
        } else {
            update(2 * v + 2, m, r, p, x);
        }
        data[v] = data[2 * v + 1] + data[2 * v + 2];
    }

    Node get(int v, int l, int r, int lq, int rq) {
        if (rq <= l || r <= lq) {
            return {};
        }
        if (lq <= l && r <= rq) {
            return data[v];
        }
        int m = (l + r) / 2;
        Node p1 = get(2 * v + 1, l, m, lq, rq);
        Node p2 = get(2 * v + 2, m, r, lq, rq);
        return p1 + p2;
    }
};


void dfs_min_dist(int v, int h, vector<vector<int>> &g, vector<int> &dist) {
    if (g[v].size() == 0) {
        dist[v] = h;
        return;
    }
    dist[v] = (int) g.size();
    for (auto u : g[v]) {
        dfs_min_dist(u, h + 1, g, dist);
        dist[v] = min(dist[v], dist[u] + 1);
    }
}

void dfs_min_tin(int v, int h, vector<vector<int>> &g, vector<int> &dist, vector<int> &id, vector<pair<int, int>> &seg,
                 vector<vector<int>> &base) {
    id[v] = (int) base[h].size();
    base[h].emplace_back(0);
    seg[v].first = (int) base[dist[v]].size();
    for (auto u : g[v]) {
        dfs_min_tin(u, h + 1, g, dist, id, seg, base);
    }
    seg[v].second = (int) base[dist[v]].size();
}
void dfs(int v, int h, vector<vector<int>> &g, vector<int> &dp, vector<int> &dist, vector<pair<int, int>> &seg,
         vector<vector<int>> &base, vector<int> &id, vector<SegmentTree> &tree) {
    for (auto u : g[v]) {
        dfs(u, h + 1, g, dp, dist, seg, base, id, tree);
        dp[v] = max(dp[v], dp[u]);
    }
    dp[v] = max(dp[v], tree[dist[v]].get(0, 0, (int) base[dist[v]].size(), seg[v].first, seg[v].second).mx + 1);
    if (g[v].size() != 0) {
        tree[h].update(0, 0, (int) base[h].size(), id[v], dp[v]);
    }
}

void solve_b() {
    int n;
    cin >> n;
    vector<vector<int>> g(n);
    for (int v = 1; v < n; ++v) {
        int p;
        cin >> p;
        p--;
        g[p].push_back(v);
    }
    vector<int> dist(n, 0);
    dfs_min_dist(0, 0, g, dist);
    vector<int> id(n, 0);
    vector<vector<int>> base(n);
    vector<pair<int, int>> seg(n);
    dfs_min_tin(0, 0, g, dist, id, seg, base);
    vector<SegmentTree> tree;
    tree.reserve(n);
    for (auto &i : base) {
        tree.emplace_back(i);
    }
    vector<int> dp (n, 0);
    dfs(0, 0, g, dp, dist, seg, base, id, tree);
    /*for (auto i : dp) {
        cout << i << ' ';
    }
    cout << '\n';*/
    cout << dp[0];
}

/*
struct Node {
    int pref, suf, seg;
    int sz;

    Node(int pref = 0, int suf = 0, int seg = 0, int sz = 0) : pref(pref), suf(suf), seg(seg), sz(sz) {}

    Node operator+(const Node &node) const {
        Node sum(pref, node.suf, max(seg, node.seg), sz + node.sz);
         if (pref == sz) {
             sum.pref += node.pref;
         }
         if (node.suf == node.sz) {
             sum.suf += suf;
         }
         sum.seg = max({sum.seg, suf + node.pref, sum.pref, sum.suf});
         return sum;
    }
};

struct Flowers {
    Node a, b;
    int x, y;

    Flowers() : a(), b(), x(0), y(0) {}

    Flowers(char c) {
        x = (c == 'a');
        y = (c == 'b');
        a = Node(x, x, x, 1);
        b = Node(y, y, y, 1);
    }

    Flowers(Node a, Node b, int x, int y) : a(a), b(b), x(x), y(y) {}

    Flowers operator+(const Flowers &f) const {
        return {a + f.a, b + f.b, x + f.x, y + f.y};
    }
};

struct SegmentTree {
    vector<Flowers> data;
    string base;

    SegmentTree(string &s) : base(s), data(4 * (int) s.size()) {
        build(0, 0, (int) s.size());
    }

    void build(int v, int l, int r) {
        if (l + 1 == r) {
            data[v] = Flowers(base[l]);
            return;
        }
        int m = (l + r) / 2;
        build(2 * v + 1, l, m);
        build(2 * v + 2, m, r);
        data[v] = data[2 * v + 1] + data[2 * v + 2];
    }

    void update(int v, int l, int r, int p) {
        if (l + 1 == r) {
            if (base[l] == 'a') {
                base[l] = 'b';
            } else {
                base[l] = 'a';
            }
            data[v] = Flowers(base[l]);
            return;
        }
        int m = (l + r) / 2;
        if (p < m) {
            update(2 * v + 1, l, m, p);
        } else {
            update(2 * v + 2, m, r, p);
        }
        data[v] = data[2 * v + 1] + data[2 * v + 2];
    }

    Flowers get(int v, int l, int r, int lq, int rq) {
        if (rq <= l || r <= lq) {
            return {};
        }
        if (lq <= l && r <= rq) {
            return data[v];
        }
        int m = (l + r) / 2;
        Flowers p1 = get(2 * v + 1, l, m, lq, rq);
        Flowers p2 = get(2 * v + 2, m, r, lq, rq);
        return p1 + p2;
    }
};

void solve_l() {
    int n;
    cin >> n;
    string s;
    cin >> s;
    SegmentTree tree(s);
    int q;
    cin >> q;
    while (q--) {
        int type;
        cin >> type;
        if (type == 1) {
            int p;
            cin >> p;
            p--;
            tree.update(0, 0, n, p);
        } else {
            int l, r;
            cin >> l >> r;
            l--;
            Flowers f = tree.get(0, 0, n, l, r);
            if (f.b.seg >= 2 || f.b.seg == 1 && f.x < f.y) {
                cout << "NO\n";
            } else {
                cout << "YES\n";
            }
        }
    }
}
*/

void solve_h() {
    ll n;
    cin>>n;
    vector<vector<ll>> dpa(n+1,vector<ll> (n+1,0)),dpb(n+1,vector<ll>(n+1,1));
    for(int i=0;i<=n;i++) {
        dpb[0][i]=0;
        dpb[1][i]=1;
    }
    dpb[1][0]=0;
    dpb[0][0]=1;
    for(int k=2;k<=n;k++) {
        for (int s = 0; s <= n; s++) {
            dpb[k][s]=0;
            for (int m = 1; s - m * k >= 0; m++) {
                dpb[k][s] += dpb[k - 1][s - m * k];
            }
            cout<<dpb[k][s]<<' ';
        }
        cout<<'\n';
    }
    cout<<'\n';
    ll ans=0;
    for(int m=0;m<=n;m++) {
        for(int k=m;k<=n;k++) {
            if(n - (m + 1) * m / 2 - (k - m) * (m + 1)>=0) {
                dpa[m][k] = dpb[n-k][n - (m + 1) * m / 2 - (k - m) * (m + 1)];
            } else {
                dpa[m][k]=0;
            }
            ans+=(m+1)*dpa[m][k];
            cout<<dpa[m][k]<<' ';
        }
        cout<<'\n';
    }
    cout<<"\n\n"<<ans;
}

int main() {
    ios_base::sync_with_stdio(false), cin.tie(0);
    //COMMENTED
    int t = 1;
    //cin >> t;
    while(t--) {
        solve_b();
    }
    return 0;
}