#include <bits/stdc++.h>

using ll = long long;
using namespace std;

struct Point {
    int x = 0, y = 0;

    Point() = default;
    Point(int x, int y) : x(x), y(y) {}
    Point(const Point &A, const Point &B) {
        x = B.x - A.x;
        y = B.y - A.y;
    }

    ll operator%(const Point &other) const {
        return x * other.y - y * other.x;
    }

    friend istream& operator>>(istream& in, Point &other) {
        return in >> other.x >> other.y;
    }
};

struct Graph {
    vector<vector<int>> g;
    int n;

    Graph(int n) : n(n) {
        g.resize(n);
    }

    void add(int u, int v) {
        g[u].push_back(v);
        g[v].push_back(u);
    }

    int get() const {
        vector<bool> used(n);

        int sz = 0;

        auto dfs = [&](auto &&dfs, int u) -> void {
            used[u] = true;
            sz += 1;
            for (int v : g[u]) {
                if (!used[v]) {
                    dfs(dfs, v);
                }
            }
        };

        int cnt = 0;

        for (int u = 0; u < n; u++) {
            if (!used[u]) {
                sz = 0;
                dfs(dfs, u);
                cnt += sz / 2;
            }
        }

        return cnt;
    }
};

Point P[200500];

vector<int> g[200500];
int dp[200500];

bool on_top(const Point &A) {
    return A.y > 0 || (A.y == 0 && A.x > 0);
}

bool comp(const Point &A, const Point &B) {
    if (on_top(A) == on_top(B)) {
        return A % B > 0;
    } else {
        return on_top(A);
    }
}

void dfs(int u, int p = -1) {
    dp[u] = (p != -1);

    for (int v : g[u]) {
        if (v != p) {
            dfs(v, u);
            dp[u] += dp[v];
        }
    }

    int m = (int)g[u].size();

    vector<tuple<Point, bool, int>> A;
    Graph G(m);

    for (int i = 0; i < m; i++) {
        int v = g[u][i];
        A.emplace_back(Point(P[u], P[v]), true, i);
        A.emplace_back(Point(P[v], P[u]), false, i);
    }

    sort(A.begin(), A.end(), [](auto A, auto B) {
        return comp(get<0>(A), get<0>(B));
    });

    vector<bool> mark(m);

    for (int i = 0; i < 2 * m; i++) {
        int j = (i + 1) % (2 * m);

        auto [P1, F1, I1] = A[i];
        auto [P2, F2, I2] = A[j];

        if (F1 != F2 && I1 != I2 && P1 % P2 == 0) {
            G.add(I1, I2);
            mark[I1] = true;
            mark[I2] = true;
        }
    }


    for (int i = 0; i < 2 * m; i++) {
        int j = (i + 1) % (2 * m);

        auto [P1, F1, I1] = A[i];
        auto [P2, F2, I2] = A[j];

        if (F1 != F2 && I1 != I2 && !mark[I1] && !mark[I2]) {
            G.add(I1, I2);
        }
    }

    dp[u] -= G.get();
}

signed main() {
    ios::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;

    for (int u = 1; u <= n; u++) {
        cin >> P[u];
    }

    for (int i = 0; i < n - 1; i++) {
        int u, v;
        cin >> u >> v;

        g[u].push_back(v);
        g[v].push_back(u);
    }

    dfs(1);

    cout << dp[1] << '\n';
    return 0;
}
